#! /usr/bin/python3

# based on usbtest.py from usbtest.py

import argparse
import json
import logging
import paho.mqtt.client as mqtt
import socket
import sys
import threading
import time

calc = False

n_invocations = 0
n_err = 0

bottom = 30

if len(sys.argv) == 2:
    bottom = float(sys.argv[1])

    print('Using %f as bottom threshold' % bottom)

def ka():
    while True:
        time.sleep(5)
        client.publish('ka', time.ctime())

def send_bfg_miner_command(command, parameter):
    s = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
    s.connect(('localhost', 4028))

    d = dict()
    d['command'] = command

    if not parameter is None:
        d['parameter'] = parameter

    s.send(json.dumps(d).encode())

    data = ''
    while True:
        try:
            newdata = s.recv(1024)
            if newdata:
                data += newdata.decode('ascii')

            else:
                break

        except socket.error as e:
            return None

    s.close()

    if data == '':
        return None

    return json.loads(data.replace('\x00', ''))

def set_workers_state(state):
    while True:
        n_pga_j = None

        try:
            n_pga_j = send_bfg_miner_command('pgacount', None)

            n_pga = n_pga_j['PGAS'][0]['Count']

            for i in range(0, n_pga):
                if send_bfg_miner_command('pgadisable' if state == False else 'pgaenable', '%d' % i) == None:
                    raise Exception('cannot send_bfg_miner_command')

            break

        except Exception as e:
            print('set_workers_state', e)
            print(n_pga_j)

            time.sleep(1)

def on_message(client, userdata, message):
    global calc

    global n_invocations
    global n_err

    try:
        t = float(message.payload)

        show = False

        if t <= bottom:
            if not calc:
                print('')
                print('Heater ON', time.ctime(), t)
                set_workers_state(True)
                show = True
                calc = True

        elif t >= 30.5:
            if calc:
                print('')
                print('Heater OFF', time.ctime(), t)
                set_workers_state(False)
                show = True
                calc = False

        if not show:
            print(time.ctime(), t)

    except Exception as e:
        print('on_message', e)

set_workers_state(False)

while True:
    try:
        client = mqtt.Client()
        client.connect('192.168.64.1', port=1883, keepalive=4, bind_address="")
        client.on_message = on_message
        client.subscribe('dh22_1_temp')

        t = threading.Thread(target=ka)
        t.daemon = True
        t.start()

        client.loop_forever()

    except Exception as e:
        print('main-loop', e)
